package es.upv.pros.tunnel.abstractClasses;

import java.util.ArrayList;
import java.util.Hashtable;

import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceReference;

import es.upv.pros.tunnel.classes.Data;
import es.upv.pros.tunnel.classes.NewDataReceived;
import es.upv.pros.tunnel.interfaces.IFactoryActuators;
import es.upv.pros.tunnel.interfaces.IActuatorAngularDimmer;
import es.upv.pros.tunnel.interfaces.IDevice;
import es.upv.pros.tunnel.interfaces.IActuatorHexDimmer;
import es.upv.pros.tunnel.interfaces.IActuatorPercentualDimmer;
import es.upv.pros.tunnel.interfaces.IActuatorPulse;
import es.upv.pros.tunnel.interfaces.ISensor;
import es.upv.pros.tunnel.interfaces.IActuatorSwitch;
import es.upv.pros.tunnel.interfaces.ITunnelReader;
import es.upv.pros.tunnel.interfaces.ITunnelWriter;
import es.upv.pros.tunnel.interfaces.IActuatorVerticalMovement;
import es.upv.pros.tunnel.interfaces.TunnelConstants;

public abstract class AbstractTunnel implements ITunnelReader, ITunnelWriter, IFactoryActuators {

	String tunnelName;
	private BundleContext bc;
	private ArrayList<Data> dataLog=new ArrayList<Data>();
	private Hashtable <String, Object> dataCache=new Hashtable<String, Object>();
	private Hashtable<String,ArrayList<ServiceReference>> mySensors;   
	private Hashtable<String,ArrayList<ServiceReference>> myActuators;  
	private Hashtable<String,Class>addressesTypes;
	private boolean isListener=false;
	private boolean isWriter=false;

	public AbstractTunnel(BundleContext context, String tunnelName) {
		this.bc=context;
		this.tunnelName=tunnelName;
		mySensors=new Hashtable<String, ArrayList<ServiceReference>>();
		myActuators=new Hashtable<String, ArrayList<ServiceReference>>();
		addressesTypes=new Hashtable<String, Class>();
	}

	public NewDataReceived formatData(String address, Object data){
		return new NewDataReceived(address, data, addressesTypes.get(address)); 
	}

	public void notifySensors(NewDataReceived data) {
		if(mySensors.containsKey(data.getAddress())){
			for(ServiceReference sr : mySensors.get(data.getAddress())){
				ISensor sensor=(ISensor)this.bc.getService(sr);
				
				//NACHO
				/*if(haveToNotify(data, sensor)){
					sensor.newDataReceived(data);
				}*/
				sensor.newDataReceived(data);
			}
			putSensorValueOnDataCache(data.getAddress(), data.getData());
		}
	}

	private boolean haveToNotify(NewDataReceived data, ISensor sensor) {
		if(dataCache.containsKey(data.getAddress())){
			if(dataCache.get(data.getAddress()).equals(data.getData())){
				if(sensor.getNotificationMode().equals(ISensor.NOTIFY_ON_CHANGE)){
					return false;
				}
			}
		}
		return true;
	}

	public void serviceChanged(ServiceEvent event) {
		String tunnelN=(String)event.getServiceReference().getProperty(IDevice.DEVICE_TUNNEL);
		if(tunnelN.equals(this.tunnelName)){
			String address=(String)event.getServiceReference().getProperty(IDevice.DEVICE_ADDRESS);
			ServiceReference sr=event.getServiceReference();
			Class dataType = null;
			try {
				dataType = Class.forName((String)event.getServiceReference().getProperty(IDevice.DEVICE_DATA_TYPE));
			} catch (ClassNotFoundException e) {

				e.printStackTrace();
			}
			if(((String)event.getServiceReference().getProperty(TunnelConstants.SERVICE_CLASS)).equals(ISensor.class.getName())){
				//SENSORS
				switch(event.getType()){
				case ServiceEvent.REGISTERED: 
					addSensor(sr, address);
					break;
				case ServiceEvent.UNREGISTERING:
					removeSensor(sr, address);
					break;
				case ServiceEvent.MODIFIED:
					removeSensor(sr, address);
					addSensor(sr, address);
					break;
				case ServiceEvent.MODIFIED_ENDMATCH:
					removeSensor(sr, address);
					break;
				}
			}
			else{
				//ACTUATOR
				switch(event.getType()){
				case ServiceEvent.REGISTERED: 
					addActuator(sr, address);
					break;
				case ServiceEvent.UNREGISTERING:
					removeActuator(sr, address);
					break;
				case ServiceEvent.MODIFIED:
					removeActuator(sr, address);
					addActuator(sr, address);
					break;
				case ServiceEvent.MODIFIED_ENDMATCH:
					removeActuator(sr, address);
					break;
				}
			}
			if(!this.addressesTypes.containsKey(address)){
				addressesTypes.put(address, dataType);
			}
		}


	}

	public void registerAsSensorListener() {
		Hashtable<String, Object> props=new Hashtable<String, Object>();
		props.put(TunnelConstants.TUNNEL_NAME, this.tunnelName);
		bc.registerService(ITunnelReader.class.getName(), this, props);

		String filter="(objectclass="+ISensor.class.getName()+")";
		try {
			this.bc.addServiceListener(this, filter);
		} catch (InvalidSyntaxException e) {
			e.printStackTrace();
		}
		sensorListening();
		isListener=true;
	}

	public void registerAsActuatorWriter() {
		Hashtable<String, Object> props=new Hashtable<String, Object>();
		props.put(TunnelConstants.TUNNEL_NAME, this.tunnelName);
		bc.registerService(ITunnelWriter.class.getName(), this, props);
		bc.registerService(IFactoryActuators.class.getName(), this, props);

		Hashtable props2=new Hashtable();
		props2.put(IFactoryActuators.TUNNEL_NAME, this.tunnelName);
		bc.registerService(IFactoryActuators.class.getName(), this, props2);

		String filter=
			"(|" +
			"(objectclass="+IActuatorAngularDimmer.class.getName()+")" +
			"(objectclass="+IActuatorHexDimmer.class.getName()+")" +
			"(objectclass="+IActuatorPercentualDimmer.class.getName()+")" +
			"(objectclass="+IActuatorPulse.class.getName()+")" +
			"(objectclass="+IActuatorSwitch.class.getName()+")" +
			"(objectclass="+IActuatorVerticalMovement.class.getName()+")" +
			")";
		try {
			this.bc.addServiceListener(this, filter);
		} catch (InvalidSyntaxException e) {
			e.printStackTrace();
		}
		isWriter=true;
	}

	public void registerAsServiceListenerAndActuatorWriter(){
		Hashtable<String, Object> props=new Hashtable<String, Object>();
		props.put(TunnelConstants.TUNNEL_NAME, this.tunnelName);
		bc.registerService(ITunnelReader.class.getName(), this, props);
		bc.registerService(ITunnelWriter.class.getName(), this, props);

		Hashtable props2=new Hashtable();
		props2.put(IFactoryActuators.TUNNEL_NAME, this.tunnelName);
		bc.registerService(IFactoryActuators.class.getName(), this, props2);
		String filter=
			"(|" +
			"(objectclass="+IActuatorAngularDimmer.class.getName()+")" +
			"(objectclass="+IActuatorHexDimmer.class.getName()+")" +
			"(objectclass="+IActuatorPercentualDimmer.class.getName()+")" +
			"(objectclass="+IActuatorPulse.class.getName()+")" +
			"(objectclass="+IActuatorSwitch.class.getName()+")" +
			"(objectclass="+IActuatorVerticalMovement.class.getName()+")" +
			"(objectclass="+ISensor.class.getName()+")"+
			")";
		try {
			this.bc.addServiceListener(this, filter);
		} catch (InvalidSyntaxException e) {
			e.printStackTrace();
		}
		sensorListening();
		isListener=isWriter=true;
	}

	public void addSensor(ServiceReference sr, String address){
		if(mySensors.containsKey(address)){
			mySensors.get(address).add(sr);
		}
		else{
			ArrayList<ServiceReference> temp=new ArrayList<ServiceReference>();
			temp.add(sr);
			mySensors.put(address, temp);
		}
	}

	public void removeSensor(ServiceReference sr, String address){
		if(mySensors.containsKey(address)){
			mySensors.get(address).remove(sr);
		}
		if(mySensors.get(address).isEmpty()){
			mySensors.remove(address);
		}
	}

	public void addActuator(ServiceReference sr, String address){
		if(myActuators.containsKey(address)){
			myActuators.get(address).add(sr);
		}
		else{
			ArrayList<ServiceReference> temp=new ArrayList<ServiceReference>();
			temp.add(sr);
			myActuators.put(address, temp);
		}
	}

	public void removeActuator(ServiceReference sr, String address){
		if(myActuators.containsKey(address)){
			myActuators.get(address).remove(sr);
		}
		if(myActuators.get(address).isEmpty()){
			myActuators.remove(address);
		}
	}

	public Class getDataType(String address) {
		if(this.addressesTypes.containsKey(address)){
			return this.addressesTypes.get(address);
		}
		return null;
	}

	public BundleContext getBundleContext(){
		return this.bc;
	}

	public void putActuatorValueOnDataCache(String address, Object data){
		dataCache.remove(address);
		dataCache.put(address, data);
		dataLog.add(new Data(address, data));
	}

	public void putSensorValueOnDataCache(String address, Object data){
		dataCache.remove(address);
		dataCache.put(address, data);
		dataLog.add(new Data(address, data));
	}

	public Object getActuatorState(String address){
		if(dataCache.containsKey(address)){
			return this.dataCache.get(address);
		}
		return null;
	}

	public void ensureConnection(){
		while(!this.isConnected()){
			System.out.println("Tunnel disconnected. Trying to reconnect.");
			try {
				this.connect();
				if(isListener){
					sensorListening();
				}
				System.out.println("Tunnel reconnected.");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	} 


	public abstract void witeFloat(String address, float value);
	public abstract void writeBoolean(String address, boolean value);
	public abstract void writeInteger(String address, int value, String scale);
	public abstract void writeString(String address, String value);
	public abstract void sensorListening();
	public abstract IActuatorAngularDimmer getAngularDimmerActuator(Hashtable props);
	public abstract IActuatorHexDimmer getHexDimmerActuator(Hashtable props);
	public abstract IActuatorPercentualDimmer getPercentualDimmerActuator(Hashtable props);
	public abstract IActuatorPulse getPulseActuator(Hashtable props);
	public abstract IActuatorSwitch getSwitchActuator(Hashtable props);
	public abstract IActuatorVerticalMovement getVMActuator(Hashtable props);	
	public abstract boolean isConnected();
	public abstract void connect() throws Exception;
	public abstract void disconnect() throws Exception;
}
