package es.upv.pros.tunnel.interfaces;

import java.util.Hashtable;

/**
 * Interface that represents a Generic device
 * @author ProS - UPV
 *
 */
public interface IDevice {
	
	public final String DEVICE_ID="es.upv.pros.IDeviceConstants.device-id";
	/**
	 * Sets the given ID to the device
	 * @param iD the new device ID
	 */
	public void setID(String iD);
	/**
	 * Returns the ID of the device as a String
	 * @return The ID of the device as a String 
	 */
	public String getID();
	
	
	public final String DEVICE_NAME="es.upv.pros.IDeviceConstants.device-name";
	/**
	 * Sets the given name to the device
	 * @param name the new device name
	 */
	public void setName(String name);
	/**
	 * Returns the name of the device as String
	 * @return The name of the device as String
	 */
	public String getName();
	
	public final String DEVICE_PROFILE="es.upv.pros.IDeviceConstants.device-profile";
	public final String DEVICE_PROFILE_EIB_ACTUATOR="es.upv.pros.IDeviceConstants.device-profile-eib-actuator";
	public final String DEVICE_PROFILE_EIB_SENSOR="es.upv.pros.IDeviceConstants.device-profile-eib-sensor";
	/**
	 * Sets the given profile to the device
	 * @param profile the new device profile
	 */
	public void setProfile(String profile);
	/**
	 * Returns the profile of the device as a String
	 * @return the profile of the device as String
	 */
	public String getProfile();
	
	public final String DEVICE_DESCRIPTION="es.upv.pros.IDeviceConstants.device-description";
	/**
	 * Sets the given description to the device
	 * @param description the new device description
	 */
	public void setDescription(String description);
	/**
	 * Returns the description of the device as a String
	 * @return the description of the device as a String
	 */
	public String getDescription();
	
	public final String DEVICE_TYPE="es.upv.pros.IDeviceConstants.device-type";
	public final String DEVICE_TYPE_SENSOR=ISensor.class.getName();
	public final String DEVICE_TYPE_PERCENTUAL_DIMMER=IActuatorPercentualDimmer.class.getName();
	public final String DEVICE_TYPE_ANGULAR_DIMMER=IActuatorAngularDimmer.class.getName();
	public final String DEVICE_TYPE_HEX_DIMMER=IActuatorHexDimmer.class.getName();
	public final String DEVICE_TYPE_VM=IActuatorVerticalMovement.class.getName();
	public final String DEVICE_TYPE_PULSE=IActuatorPulse.class.getName();
	public final String DEVICE_TYPE_SWITCH=IActuatorSwitch.class.getName();
	/**
	 * Sets the given type to the device <br/>The type must be one of the DEVICE_TYPE_... constants
	 * @param deviceType the new device type
	 */
	public void setDeviceType(String deviceType);
	/**
	 * Returns the type of the device as a String
	 * @return the type of the device as a String
	 */
	public String getDeviceType();
	
	public final String DEVICE_ADDRESS="es.upv.pros.IDeviceConstants.device-address";
	/**
	 * Sets the given address to the device
	 * @param address the new device address
	 */
	public void setAddress(String address);
	/**
	 * Returns the address of the device as a String
	 * @return the address of the device as a String
	 */
	public String getAddress();
	
	
	public final String DEVICE_LOCATION="es.upv.pros.IDeviceConstants.device-location";
	/**
	 * Sets the given location to the device
	 * @param location the new device location
	 */
	public void setLocation(String location);
	/**
	 * Returns the location of the device as a String
	 * @return the location of the device as a String
	 */
	public String getLocation();
	
	public final String DEVICE_MANUFACTURER="es.upv.pros.IDeviceConstants.device-manufacturer";
	/**
	 * Sets the manufacturer of the device
	 * @param manufacturer the new device location
	 */
	public void setManufacturer(String manufacturer);
	/**
	 * Returns the manufacturer of the device as a String
	 * @return the manufacturer of the device as a String
	 */
	public String getManufacturer();
	
	public final String DEVICE_SERIAL_NUMBER="es.upv.pros.IDeviceConstants.device-serial-number";
	/**
	 * Sets the serial number of the device
	 * @param serialNumber the new serial number location
	 */
	public void setSerialNumber(String serialNumber);
	/**
	 * Returns the serial number of the device as a String
	 * @return the serial number of the device as a String
	 */
	public String getSerialNumber();
	
	public final String DEVICE_MODEL="es.upv.pros.IDeviceConstants.device-model";
	/**
	 * Sets the model of the device
	 * @param model the new manufacturer location
	 */
	public void setModel(String model);
	/**
	 * Returns the model of the device as a String
	 * @return the model of the device as a String
	 */
	public String getModel();
	
	public final String DEVICE_TUNNEL="es.upv.pros.IDeviceConstants.device-tunnel";
	/**
	 * Sets the tunnel of the device
	 * @param tunnel the new device tunnel
	 */
	public void setTunnel(String tunnel);
	/**
	 * Returns the tunnel of the device as a String
	 * @return the tunnel of the device as a String
	 */
	public String getTunnel();
	
	public final String DEVICE_DATA_TYPE="es.upv.pros.IDeviceConstants.device-data-type";
	/**
	 * Sets the data type of the device
	 * @param dataType the new device data type
	 */
	public void setDataType(String dataType);
	/**
	 * Returns the data type of the device as a String
	 * @return the data type of the device as a String
	 */
	public String getDataType();
	
	public final String DEVICE_OTHER_PROPERTIES="es.upv.pros.IDeviceConstants.device-other-properties";
	/**
	 * Sets the other properties of the device
	 * @param otherProperties the new device other properties
	 */
	public void setOtherProperties(Hashtable<String, Object> otherProperties);
	/**
	 * Returns the other properties hashtable of the device
	 * @return the the other properties hashtable of the device
	 */
	public Hashtable<String, Object> getOtherProperties();
	
	public Object getState();
	
	public final String DEVICE_SUBTYPE="es.upv.pros.IDeviceConstants.device-sub-type";
	
	/**
	 * Sets the subtype of the device
	 * @param subtype the new device subtype
	 */
	public void setSubType(String subType);
	/**
	 * Returns the other properties hashtable of the device
	 * @return the the other properties hashtable of the device
	 */
	public String getSubType();
	
}
