package es.upv.pros.tunnel.interfaces;

import java.util.Hashtable;

/**
 * Service Objects registered under the IActuatorProvider are expected to provide 
 * Actuator for a specific tunnel.
 * @author ProS - UPV
 *
 */

public interface IFactoryActuators {

	public static final String TUNNEL_NAME="es.upv.pros.IActuatorProviderConstants.Tunnel-name";
	
	/**
	 * This method returns a IAngularDimmerActuator for the given tunnel and formatted with the parameter props
	 * @param props The IAngularDimmerActuator properties
	 * @return The IAngularDimmerActuator formatted.
	 */
	public IActuatorAngularDimmer getAngularDimmerActuator(Hashtable props);
	
	/**
	 * This method returns a IHexDimmerActuator for the given tunnel and formatted with the parameter props
	 * @param props The IHexDimmerActuator properties
	 * @return The IHexDimmerActuator formatted.
	 */
	public IActuatorHexDimmer getHexDimmerActuator(Hashtable props);
	
	/**
	 * This method returns a IPercentualDimmerActuator for the given tunnel and formatted with the parameter props
	 * @param props The IPercentualDimmerActuator properties
	 * @return The IPercentualDimmerActuator formatted.
	 */
	public IActuatorPercentualDimmer getPercentualDimmerActuator(Hashtable props);
	
	/**
	 * This method returns a ISwitchActuator for the given tunnel and formatted with the parameter props
	 * @param props The ISwitchActuator properties
	 * @return The ISwitchActuator formatted.
	 */
	public IActuatorSwitch getSwitchActuator(Hashtable props);
	
	
	/**
	 * This method returns a IPulseActuator for the given tunnel and formatted with the parameter props
	 * @param props The IPulseActuator properties
	 * @return The IPulseActuator formatted.
	 */
	public IActuatorPulse getPulseActuator(Hashtable props);
	
	/**
	 * This method returns a IVMActuator for the given tunnel and formatted with the parameter props
	 * @param props The IVMActuator properties
	 * @return The IVMActuator formatted.
	 */
	public IActuatorVerticalMovement getVMActuator(Hashtable props);
	
	public IDevice getNewDevice(Hashtable props);
	
	
}
