package es.upv.pros.tunnel.interfaces;

import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;

import es.upv.pros.tunnel.classes.NewDataReceived;

/**
 * Services registered under the ITunnelReader interface, allow access to the sensors of a given tunnel.
 * @author ProS - UPV
 *
 */
public interface ITunnelReader extends ServiceListener {
	
	/**
	 * This method is called when a new data is received by a sensor.
	 * @param data The formatted new data received
	 */
	public void notifySensors(NewDataReceived data);
	
	/**
	 *This method register the service as ITunnelReades 
	 */
	public void registerAsSensorListener();
	
	/**
	 * This method adds a given sensor to a collection of sensors in a tunnel
	 * @param sr The reference of the sensor service
	 * @param address The sensor address
	 */
	public void addSensor(ServiceReference sr, String address);
	
	/**
	 * This method removes a given sensor from a collection of sensors in a tunnel
	 * @param sr The reference of the sensor service
	 * @param address The sensor address
	 */
	public void removeSensor(ServiceReference sr, String address);
	
	/**
	 * This method formats the data received by one sensor to an instance of NewDataReceived
	 * @param address The address of the sensor
	 * @param data The data received
	 * @return The data formatted
	 */
	public NewDataReceived formatData(String address, Object data);
	
	/**
	 * This method are called to implement the sensor access of a given tunnel
	 */
	public void sensorListening();
	
	public void putSensorValueOnDataCache(String address, Object data);

}
