package es.upv.pros.tunnel.interfaces;

import org.osgi.framework.ServiceReference;

/**
 * Services registered under the ITunnelWriter interface, allow access to the actuator of a given tunnel.
 * @author ProS - UPV
 *
 */
public interface ITunnelWriter {
	
	/**
	 * Sends the Integer provided value to a actuator
	 * @param address the actuator address
	 * @param value the int value
	 * @param scale the scale of the device (dimmer)
	 */
	public void writeInteger(String address, int value, String scale);
	/**
	 * Sends the boolean provided value to a actuator
	 * @param address the actuator address
	 * @param value the boolean value
	 */
	public void writeBoolean(String address, boolean value);
	/**
	 * Sends the float provided value to a actuator
	 * @param address the actuator address
	 * @param value the float value
	 */
	public void witeFloat(String address, float value);
	/**
	 * Sends the string provided value to a actuator
	 * @param address the actuator address
	 * @param value the string value
	 */
	public void writeString(String address, String value);
	/**
	 * This method adds a given actuator to a collection of actuators in a tunnel
	 * @param sr The reference of the actuator service
	 * @param address The actuator address
	 */
	public void addActuator(ServiceReference sr, String address);
	/**
	 * This method removes a given actuator from a collection of actuators in a tunnel
	 * @param sr The reference of the actuator service
	 * @param address The actuator address
	 */
	public void removeActuator(ServiceReference sr, String address);
	public void registerAsActuatorWriter();
	
	public Object getActuatorState(String address);

	public void putActuatorValueOnDataCache(String address, Object data);
	
}
