package es.upv.pros.tunnel.tools;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;

import es.upv.pros.tunnel.interfaces.IFactoryActuators;
import es.upv.pros.tunnel.interfaces.IActuatorAngularDimmer;
import es.upv.pros.tunnel.interfaces.IDevice;
import es.upv.pros.tunnel.interfaces.IActuatorHexDimmer;
import es.upv.pros.tunnel.interfaces.IActuatorPercentualDimmer;
import es.upv.pros.tunnel.interfaces.IActuatorPulse;
import es.upv.pros.tunnel.interfaces.IActuatorSwitch;
import es.upv.pros.tunnel.interfaces.IActuatorVerticalMovement;



/**
 * This class provide some auxiliar methods for managing the tunnel
 * @author ProS - UPV
 *
 */
public final class Tools {



	/**
	 * Registers on the OSGi register a new Driver with the properties provided by the user
	 * @param context The OSGi Bundle Context
	 * @param props the properties of the new Driver
	 * @return The ServiceRegistration of the registered driver
	 */
	public static ServiceRegistration createAndRegisterDriver(BundleContext context, Hashtable<String, Object> props){
		try {
			
			IDevice device=(IDevice)Tools.createDriverObject(context, props);
			
			props.put(org.osgi.framework.Constants.SERVICE_PID, device.getID());
			return context.registerService(device.getDeviceType(), device, props);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Registers on the OSGi register the given driver with the properties provided by the user
	 * @param context The OSGi Bundle Context
	 * @param device The driver
	 * @param props The driver properties 
	 * @return The ServiceRegistration of the registered driver
	 */
	public static ServiceRegistration registerDriver(BundleContext context, IDevice device, Hashtable<String, Object> props){
		try {
			return context.registerService(device.getDeviceType(), device, props);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}


	/**
	 * Returns a new Driver with the properties provided by the user
	 * @param context The OSGi Bundle Context
	 * @param props the properties of the new Driver
	 * @return The new Driver
	 * @throws Exception 
	 */
	public static Object createDriverObject(BundleContext context, Hashtable<String, Object> props) throws Exception  {
		props=configureProperties(props);
		ServiceReference[] sr=context.getServiceReferences(IFactoryActuators.class.getName(), "("+IFactoryActuators.TUNNEL_NAME+"="+props.get(IDevice.DEVICE_TUNNEL)+")");
		if(sr!=null&&sr.length>0){
			IFactoryActuators myProvider=(IFactoryActuators) context.getService(sr[0]);
			
			return myProvider.getNewDevice(props);

		}
		else{
			throw new Exception("NO TUNNEL");
		}
	}

	/**
	 * Constructs a LDAP Filter with the fields provided by the fields parameter
	 * @param fields The fields of the filter
	 * @return a String with the filter
	 */
	public static String constructFilter(Hashtable<String,Object> fields){
		String res="";
		int fieldNumber=0;
		Set<String> keys=fields.keySet();
		for (String key : keys) {
			if(!key.equals(IDevice.DEVICE_OTHER_PROPERTIES)){
				if(!fields.get(key).equals("")){
					res+="("+key+"="+fields.get(key)+")";
					fieldNumber++;
				}
			}
		}
		if(fieldNumber>1){
			res="(&"+res+")";
		}
		return res;
	}

	/**
	 * Configure the properties Hashtable adding the arguments that are not added by the user
	 * @param props The properties Hashtable provided by the user
	 * @return The complete properties Hashmap
	 * @throws Exception when a required argument has not been provided by the user
	 */
	private static Hashtable<String, Object> configureProperties(Hashtable<String, Object> props) throws Exception{
		if(!props.containsKey(IDevice.DEVICE_ADDRESS)){
			throw new Exception("NO ADDRESS SPECIFIED");
		}
		if(!props.containsKey(IDevice.DEVICE_ID)){
			throw new Exception("NO ID SPECIFIED");
		}
		if(!props.containsKey(IDevice.DEVICE_TUNNEL)){
			throw new Exception("NO TUNNEL SPECIFIED");
		}
		if(!props.containsKey(IDevice.DEVICE_TYPE)){
			throw new Exception("NO TYPE SPECIFIED");
		}

		if(!props.containsKey(IDevice.DEVICE_DATA_TYPE)){
			props.put(IDevice.DEVICE_DATA_TYPE, Object.class.getName());
		}
		if(!props.containsKey(IDevice.DEVICE_DESCRIPTION)){
			props.put(IDevice.DEVICE_DESCRIPTION, "");
		}
		if(!props.containsKey(IDevice.DEVICE_LOCATION)){
			props.put(IDevice.DEVICE_LOCATION, "");
		}
		if(!props.containsKey(IDevice.DEVICE_MANUFACTURER)){
			props.put(IDevice.DEVICE_MANUFACTURER, "");
		}
		if(!props.containsKey(IDevice.DEVICE_MODEL)){
			props.put(IDevice.DEVICE_MODEL, "");
		}
		if(!props.containsKey(IDevice.DEVICE_NAME)){
			props.put(IDevice.DEVICE_NAME, "");
		}
		if(!props.containsKey(IDevice.DEVICE_OTHER_PROPERTIES)){
			props.put(IDevice.DEVICE_OTHER_PROPERTIES, new Hashtable<String, Object>());
		}
		if(!props.containsKey(IDevice.DEVICE_PROFILE)){
			props.put(IDevice.DEVICE_PROFILE, "");
		}
		if(!props.containsKey(IDevice.DEVICE_SERIAL_NUMBER)){
			props.put(IDevice.DEVICE_SERIAL_NUMBER, "");
		}
		return props;
	}

	/**
	 * Unregisters the given registered drivers
	 * @param drivers the registered drivers
	 */
	public static void unregisterDrivers(Vector<ServiceRegistration> drivers){
		for (ServiceRegistration serviceRegistration : drivers) {
			serviceRegistration.unregister();
		}
	}

}
