package es.upv.pros.tunnel.calimero.impl;

import java.util.Hashtable;

import org.osgi.framework.BundleContext;

import tuwien.auto.calimero.CloseEvent;
import tuwien.auto.calimero.DetachEvent;
import tuwien.auto.calimero.FrameEvent;
import tuwien.auto.calimero.GroupAddress;
import tuwien.auto.calimero.exception.KNXFormatException;
import tuwien.auto.calimero.link.KNXNetworkLink;
import tuwien.auto.calimero.link.KNXNetworkLinkIP;
import tuwien.auto.calimero.link.KNXNetworkMonitor;
import tuwien.auto.calimero.link.KNXNetworkMonitorIP;
import tuwien.auto.calimero.link.event.NetworkLinkListener;
import tuwien.auto.calimero.link.medium.TPSettings;
import tuwien.auto.calimero.process.ProcessCommunicator;
import tuwien.auto.calimero.process.ProcessCommunicatorImpl;
import tuwien.auto.calimero.process.ProcessEvent;
import tuwien.auto.calimero.process.ProcessListenerEx;
import tuwien.auto.calimero.tools.NetworkMonitor;

import es.upv.pros.tunnel.abstractClasses.AbstractTunnel;
import es.upv.pros.tunnel.classes.NewDataReceived;
import es.upv.pros.tunnel.interfaces.IActuatorAngularDimmer;
import es.upv.pros.tunnel.interfaces.IDevice;
import es.upv.pros.tunnel.interfaces.IActuatorHexDimmer;
import es.upv.pros.tunnel.interfaces.IActuatorPercentualDimmer;
import es.upv.pros.tunnel.interfaces.IActuatorPulse;
import es.upv.pros.tunnel.interfaces.IActuatorSwitch;
import es.upv.pros.tunnel.interfaces.IActuatorVerticalMovement;

public class tunnelCalimero extends AbstractTunnel {

	private static String ADDRESS="158.42.185.5";
	ProcessCommunicator tunnel;
	KNXNetworkLink link;
	public tunnelCalimero(BundleContext context, String tunnelName) {
		super(context, tunnelName);
		try {
			link = new KNXNetworkLinkIP(ADDRESS, TPSettings.TP1);
			tunnel=new ProcessCommunicatorImpl(link);
			registerAsServiceListenerAndActuatorWriter();

		} catch (Exception e) {

			e.printStackTrace();
		}


	}

	@Override
	public void sensorListening() {
		ensureConnection();
		tunnel.addProcessListener(new ProcessListenerEx() {
			public void groupWrite(ProcessEvent arg0) {
				NewDataReceived data=null;
				Class c=getDataType(arg0.getDestination().toString());
				if (c!=null){
					if(c.equals(Boolean.class)){
						try {
							data=formatData(arg0.getDestination().toString(), asBool(arg0));
						} catch (KNXFormatException e) {
							e.printStackTrace();
						}
					}
					else if(c.equals(Float.class)){
						try {
							data=formatData(arg0.getDestination().toString(), asFloat(arg0));
						} catch (KNXFormatException e) {

							e.printStackTrace();
						}
					}
					else if(c.equals(String.class)){
						try {
							data=formatData(arg0.getDestination().toString(), asString(arg0));
						} catch (KNXFormatException e) {
							e.printStackTrace();
						}
					}
					else if(c.equals(Integer.class)){
						try {
							data=formatData(arg0.getDestination().toString(),Integer.valueOf(this.asString(arg0)));
						} catch (KNXFormatException e) {
							e.printStackTrace();
						}
					}

					if(data!=null){
						notifySensors(data);
					}
				}

			}



			public void detached(DetachEvent arg0) {}

			@Override
			public void groupReadRequest(ProcessEvent arg0) {}

			@Override
			public void groupReadResponse(ProcessEvent arg0) {}
		});


	}

	@Override
	public void witeFloat(String address, float value) {
		try {
			ensureConnection();
			tunnel.write(new GroupAddress(address), value);
		} catch (KNXFormatException e) {

			e.printStackTrace();
		} catch (Exception e) {

			e.printStackTrace();
		}

	}

	@Override
	public void writeBoolean(String address, boolean value) {
		try {
			ensureConnection();
			tunnel.write(new GroupAddress(address), value);
		} catch (KNXFormatException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	@Override
	public void writeInteger(String address, int value, String scale) {
		try {
			ensureConnection();
			tunnel.write(new GroupAddress(address), value,scale);
		} catch (KNXFormatException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	@Override
	public void writeString(String address, String value) {
		try {
			ensureConnection();
			tunnel.write(new GroupAddress(address), value);
		} catch (KNXFormatException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	@Override
	public IActuatorAngularDimmer getAngularDimmerActuator(Hashtable props) {		
		Hashtable otherProps=(Hashtable) props.get(IDevice.DEVICE_OTHER_PROPERTIES);
		props.remove(IDevice.DEVICE_OTHER_PROPERTIES);
		IActuatorAngularDimmer act= new CalimeroAngularDimmer(this.getBundleContext(),props, otherProps);
		return act;
	}

	@Override
	public IActuatorHexDimmer getHexDimmerActuator(Hashtable props) {
		Hashtable otherProps=(Hashtable) props.get(IDevice.DEVICE_OTHER_PROPERTIES);
		props.remove(IDevice.DEVICE_OTHER_PROPERTIES);
		IActuatorHexDimmer act=new CalimeroHexDimmer(this.getBundleContext(),props,otherProps);
		return act;
	}

	@Override
	public IActuatorPercentualDimmer getPercentualDimmerActuator(Hashtable props) {
		Hashtable otherProps=(Hashtable) props.get(IDevice.DEVICE_OTHER_PROPERTIES);
		props.remove(IDevice.DEVICE_OTHER_PROPERTIES);
		IActuatorPercentualDimmer act= new CalimeroPercentualDimmer(this.getBundleContext(),props,otherProps);
		return act;
	}

	@Override
	public IActuatorPulse getPulseActuator(Hashtable props) {
		Hashtable otherProps=(Hashtable) props.get(IDevice.DEVICE_OTHER_PROPERTIES);
		props.remove(IDevice.DEVICE_OTHER_PROPERTIES);
		IActuatorPulse act= new CalimeroPulse(this.getBundleContext(),props,otherProps);
		return act;
	}

	@Override
	public IActuatorSwitch getSwitchActuator(Hashtable props) {
		Hashtable otherProps=(Hashtable) props.get(IDevice.DEVICE_OTHER_PROPERTIES);
		props.remove(IDevice.DEVICE_OTHER_PROPERTIES);
		IActuatorSwitch act= new CalimeroSwitch(this.getBundleContext(),props,otherProps);
		return act;
	}

	@Override
	public IActuatorVerticalMovement getVMActuator(Hashtable props) {
		Hashtable otherProps=(Hashtable) props.get(IDevice.DEVICE_OTHER_PROPERTIES);
		props.remove(IDevice.DEVICE_OTHER_PROPERTIES);
		IActuatorVerticalMovement act= new CalimeroVM(this.getBundleContext(),props,otherProps);
		return act;
	}

	public IDevice getNewDevice(Hashtable props) {
		if(props.get(IDevice.DEVICE_TYPE).equals(IDevice.DEVICE_TYPE_SWITCH)){
			return getSwitchActuator(props);
		}
		if(props.get(IDevice.DEVICE_TYPE).equals(IDevice.DEVICE_TYPE_VM)){
			return getVMActuator(props);
		}
		if(props.get(IDevice.DEVICE_TYPE).equals(IDevice.DEVICE_TYPE_PULSE)){
			return getPulseActuator(props);
		}
		if(props.get(IDevice.DEVICE_TYPE).equals(IDevice.DEVICE_TYPE_ANGULAR_DIMMER)){
			return getAngularDimmerActuator(props);
		}
		if(props.get(IDevice.DEVICE_TYPE).equals(IDevice.DEVICE_TYPE_HEX_DIMMER)){
			return getHexDimmerActuator(props);
		}
		if(props.get(IDevice.DEVICE_TYPE).equals(IDevice.DEVICE_TYPE_PERCENTUAL_DIMMER)){
			return getPercentualDimmerActuator(props);
		}
		return null;

	}



	@Override
	public void connect() throws Exception {
		try {
			link=new KNXNetworkLinkIP(ADDRESS, TPSettings.TP1);
			tunnel=new ProcessCommunicatorImpl(link);
		} catch (Exception e) {
			
		}
	} 

	@Override
	public void disconnect() throws Exception {
		tunnel=null;
		link.close();

	}



	@Override
	public boolean isConnected() {
		if(link!=null){
			return link.isOpen();
		}
		return false;
	}
}
